#include "FreeRTOS.h"
#include "efm32_adc.h"
#include "efm32_cmu.h"
#include "efm32_rtc.h"
#include "adc.h"
#include "semphr.h"

static void ADCConfig(void);
void initADC();
/* Sempahore and queue used for light task */
xSemaphoreHandle xLightSemaphore;
xQueueHandle xQueueLight;

/***************************************************************************//**
* @brief
*   Configure ADC usage for measuring ambient light value.
*******************************************************************************/
static void ADCConfig(void)
{
  ADC_Init_TypeDef       init       = ADC_INIT_DEFAULT;
  ADC_InitSingle_TypeDef singleInit = ADC_INITSINGLE_DEFAULT;
  ADC_InitScan_TypeDef scanInit = ADC_INITSCAN_DEFAULT;
  
  /* Init common settings for both single conversion and scan mode */
  init.timebase = ADC_TimebaseCalc(0);

  /* Set ADC clock to 7 MHz, use default HFPERCLK */
  init.prescale = ADC_PrescaleCalc(4000000, 0);

  /* WARMUPMODE must be set to Normal according to ref manual before */
  ADC_Init(ADC0, &init);

  /* Init for single conversion use, measure light sensor with 2.5 reference. */
  singleInit.reference = adcRef2V5;
  singleInit.input     = adcSingleInpCh5;
  singleInit.resolution = adcRes8Bit;
  singleInit.acqTime = adcAcqTime32;

  ADC_InitSingle(ADC0, &singleInit);
  
  /* Init for scan sequence use (accelerometer X, Y and Z axis). */ 
  
  scanInit.reference = adcRefVDD;
  scanInit.input = ADC_SCANCTRL_INPUTMASK_CH2 |
                   ADC_SCANCTRL_INPUTMASK_CH3 |
                   ADC_SCANCTRL_INPUTMASK_CH4;
  ADC_InitScan(ADC0, &scanInit);
  
  /* Clear all pending interrupts */
  ADC0->IFC = _ADC_IFC_MASK;
  ADC0->IEN =  ADC_IEN_SCAN|ADC_IEN_SINGLE;
    
  NVIC_SetPriority(ADC0_IRQn,6);
	
  /* Enable interrupt in Cortex Core */
  NVIC_ClearPendingIRQ(ADC0_IRQn);
  NVIC_EnableIRQ(ADC0_IRQn);
}

/* Init ADC either for single (Light sensor) and SCAN mode (Accelerometer)*/
void initADC(){

  /* Enable clocks required */
  CMU_ClockEnable(cmuClock_HFPER, true);
  CMU_ClockEnable(cmuClock_ADC0, true);
  ADCConfig();

}

/* ADC HANDLER */
void ADC0_IRQHandler(void)
{
  portBASE_TYPE sample;
  portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;
  
  /* Interrupt triggered by light sensor */
  if ( ADC0->IF & ADC_IF_SINGLE ){
      
      /* Clear ADC Single Interrupt */
      ADC_IntClear(ADC0, ADC_IF_SINGLE);
      
      
      /* Get the value just sampled */
      sample = ADC_DataSingleGet(ADC0);
      
      /* Send the sampled value back to the LightTask. This operation is not
         as efficient as using the DMA because the system is forced to stay in 
         EM1 instead of EM2.
       */
      xQueueSendFromISR(xQueueLight,&sample,&xHigherPriorityTaskWoken);
      
      /* Unlock the semaphore that was keeping blocked "LightTask". After this, 
         the task will be immediately executed.
       */
      xSemaphoreGiveFromISR(xLightSemaphore,&xHigherPriorityTaskWoken);
  }
  
  /* Interrupt triggered by Accelerometer */
  if ( ADC0->IF & ADC_IF_SCAN ){
      
      /* Clear ADC Scan Interrupt */
      ADC_IntClear(ADC0, ADC_IF_SCAN);
      
      /* Unlock EM1 locked for ADC in GPIO*/
      xPMUnlockSleepModeFromISR(1);
      
      /* Right Now the DMA might still sending some other data. I can run in EM2 */
      xPMLockSleepModeFromISR(2);
  }
  
  /* Disable ADC in order to avoid useless sampling */
  ADC0->CMD=ADC_CMD_SINGLESTOP;
  
    /* Disable ADC in order to avoid useless sampling */
  ADC0->CMD=ADC_CMD_SCANSTOP;
 vPMTicklessFromISR(xHigherPriorityTaskWoken);

}