/*
    FreeRTOS V7.0.0 - Copyright (C) 2011 Real Time Engineers Ltd.
	

	FreeRTOS supports many tools and architectures. V7.0.0 is sponsored by:
	Atollic AB - Atollic provides professional embedded systems development 
	tools for C/C++ development, code analysis and test automation.  
	See http://www.atollic.com
	

    ***************************************************************************
     *                                                                       *
     *    FreeRTOS tutorial books are available in pdf and paperback.        *
     *    Complete, revised, and edited pdf reference manuals are also       *
     *    available.                                                         *
     *                                                                       *
     *    Purchasing FreeRTOS documentation will not only help you, by       *
     *    ensuring you get running as quickly as possible and with an        *
     *    in-depth knowledge of how to use FreeRTOS, it will also help       *
     *    the FreeRTOS project to continue with its mission of providing     *
     *    professional grade, cross platform, de facto standard solutions    *
     *    for microcontrollers - completely free of charge!                  *
     *                                                                       *
     *    >>> See http://www.FreeRTOS.org/Documentation for details. <<<     *
     *                                                                       *
     *    Thank you for using FreeRTOS, and thank you for your support!      *
     *                                                                       *
    ***************************************************************************


    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    >>>NOTE<<< The modification to the GPL is included to allow you to
    distribute a combined work that includes FreeRTOS without being obliged to
    provide the source code for proprietary components outside of the FreeRTOS
    kernel.  FreeRTOS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "croutine.h"
#include "task.h"
#include "PowerManager.h"
#include "semphr.h"

/* External modules */
#include "adc.h"
#include "dma.h"
#include "leuart.h"
#include "gpio.h"

/* BSP */
#include "dvk.h"

/* Board drivers */
#include "segmentlcd.h"
#include "segmentfont.h"
#include "lcdtest.h"
#include "i2cdrv.h"
#include "tempsens.h"

/* EFM32 library */
#include "chip.h"
#include "efm32.h"
#include "efm32_chip.h"
#include "efm32_cmu.h"
#include "efm32_emu.h"
#include "efm32_adc.h"
#include "efm32_lcd.h"
#include "efm32_dma.h"

/* Task priorities. */
#define IDLE			( tskIDLE_PRIORITY )
#define TEST1_PRIORITY          IDLE+1
#define TEST2_PRIORITY          IDLE+2
#define mainSTACK_SIZE   	(configMINIMAL_STACK_SIZE + 50)

/* Number of samples taken from accelerometer. */
#define ACCEL_SAMPLES               3

int errataShift = 0;

/* Tasks function */
void vLightLevel( void *pvParameters);

void vAccelerometer( void *pvParameters);

void vTemperature( void *pvParameters);

static void vLeuartSend( void *pvParameters);

/* Functions used inside tasks */
void inithw( void );

void setupSWO( void );

void vSampleLCD(portBASE_TYPE sample);

void SetInitialLCD();

void temperatureUpdateLCD(TEMPSENS_Temp_TypeDef *temp);

int main( void )
{
	/* Perform the necessary hardware configuration. */
	inithw();
        
        xQueueLight = xQueueCreate(1, sizeof(portBASE_TYPE));
	
        xTaskCreate( vLightLevel, "LightMeasurement", mainSTACK_SIZE, NULL,TEST1_PRIORITY, NULL );
        
       // xTaskCreate( vAccelerometer, "Accelerometer", mainSTACK_SIZE, NULL,TEST2_PRIORITY, NULL );
        
        xTaskCreate( vTemperature, "Temperature", mainSTACK_SIZE, NULL,TEST1_PRIORITY, NULL );

       // xTaskCreate( vLeuartSend, "RS232B", mainSTACK_SIZE, NULL,TEST2_PRIORITY, NULL );
        
        vSemaphoreCreateBinary(xLightSemaphore);
        //vSemaphoreCreateBinary(xAccelSemaphore);
        
        /* Start the scheduler. */
	vTaskStartScheduler();
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* Use the idle task to place the CPU into a low power mode.  Greater power
	saving could be achieved by not including any demo tasks that never block. */
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will be called if a task overflows its stack, if
	configCHECK_FOR_STACK_OVERFLOW != 0.  It might be that the function
	parameters have been corrupted, depending on the severity of the stack
	overflow.  When this is the case pxCurrentTCB can be inspected in the
	debugger to find the offending task. */
	for( ;; );
}


static void vLightLevel( void *pvParameters)
{
 portTickType xLastWakeTime;
 portBASE_TYPE sample,i;
 
  xSemaphoreTake(xLightSemaphore,0);
  xLastWakeTime = xTaskGetTickCount();
  
  for(;;)
  {
       /* Set sleep mode 1 for idle task because I need to use ADC */
      xPMSetSleepMode(EM1);
      
      /* Enable the initial characters on the screen if the light was too low */
      SetInitialLCD();
    
      /* Start ADC conversion to measure the light value */
      ADC_Start(ADC0, adcStartSingle);
      
      /* Block task until the ADC sends back the light value */
      if (xSemaphoreTake(xLightSemaphore,portMAX_DELAY) == pdTRUE){
          
          /* Wait 100 ticks if the value is not still available in the queue */ 
          xQueueReceive(xQueueLight,&sample,100);
          
          /* Fill the LCD ring according the sample value */
          vSampleLCD(sample);
      }
      
      xSemaphoreTake(xLightSemaphore,0);
      
      /* I can use sleep mode 2 because I do not need ADC anymore */
      xPMSetSleepMode(EM2);
       
      vTaskDelayUntil(&xLastWakeTime, (12000/portTICK_RATE_MS));
  }
}
/*-----------------------------------------------------------*/

/* This task is not used because the accelerometer data is sent when
   SW1 is being pressed
 */

static void vAccelerometer( void *pvParameters){

   portTickType xLastWakeTime;
   portBASE_TYPE samples[ACCEL_SAMPLES] = 0;
   
   xSemaphoreTake(xAccelSemaphore,0);
   xLastWakeTime = xTaskGetTickCount();
   
  for(;;)
  {
    
    xPMSetSleepMode(EM1);

    /* Enable the initial characters on the screen if the light was too low */
    SetInitialLCD();
    
    /* Init DMA for accelerometer */
    DMAInitAccel();   
    
    /* Enable transfer data when ADC comepletes the scan operation */
    DMA_ActivateBasic(ACCEL_DMA_CHANNEL, true, false, samples,
                     (void *)((uint32_t)&(ADC0->SCANDATA)),ACCEL_SAMPLES - 1);
    
    /* Enable ADC scan operation for X, Y, Z */
    ADC_Start(ADC0, adcStartScan);
    
    /* Block task until the ADC sends back the Accelerometer value */
    if (xSemaphoreTake(xAccelSemaphore,portMAX_DELAY) == pdTRUE){

        printf("Sample %d\n",samples[1]);
        xPMSetSleepMode(EM2);
    }
    
     xSemaphoreTake(xAccelSemaphore,0);
    
    vTaskDelayUntil(&xLastWakeTime, (150/portTICK_RATE_MS));
  }
}

/* Evaluate the temperature of Board every 60 seconds*/
static void vTemperature( void *pvParameters){

  portTickType xLastWakeTime;
  TEMPSENS_Temp_TypeDef temp;
  TEMPSENS_Temp_TypeDef prevTemp;
  
  xLastWakeTime = xTaskGetTickCount();

  /* Define previous temp to invalid, just to ensure update first time */
  prevTemp.i = 1000;
  prevTemp.f = 0;
  
  for(;;)
  {
    
    xPMSetSleepMode(2);  
    
    /* Enable the initial characters on the screen if the light was too low */
    SetInitialLCD();
    
    if (TEMPSENS_TemperatureGet(I2C0, TEMPSENS_DVK_ADDR, &temp) == 0){
      
        /* Update LCD display if any change */
        if ( (prevTemp.i != temp.i) || (prevTemp.f != temp.f) )
           temperatureUpdateLCD(&temp);

        /* Save current temp for next measurement */
        prevTemp = temp;
    }
       
    vTaskDelayUntil(&xLastWakeTime, (60000/portTICK_RATE_MS));
  }
}
/* Task not currently used because the data are sent when the user press SW1 */
static void vLeuartSend( void *pvParameters){

  portTickType xLastWakeTime;
  char samples[10];
  xLastWakeTime = xTaskGetTickCount();

  for(;;)
  {
    /* Enable the initial characters on the screen if the light was too low */
    SetInitialLCD();
    
    /* Set DMA configuration for LEUART */
    DMAInitLeuart();
   
     /* Starting the transfer between Memory and LEUART by means of DMA */
    DMA_ActivateBasic(0,  
                    true,                     
                    false,                 
                    (void *)((uint32_t)&(LEUART1->TXDATA)),           
                    samples,                       
                    strlen(samples));               
      
    xPMSetSleepMode(2);
    
    vTaskDelayUntil(&xLastWakeTime, (1000/portTICK_RATE_MS));
  }
}

/* Init all peripherals and devices present on the board */
void inithw(void){
  
  bool vboost = false;  
  SYSTEM_ChipRevision_TypeDef chipRev;
  I2C_Init_TypeDef i2cInit = I2C_INIT_DEFAULT;
  
    /* Chip revision alignment and errata fixes */
    CHIP_init();
    
    /* ADC errata for rev B when using VDD as reference, need to multiply */
    /* result by 2 */
    SYSTEM_ChipRevisionGet(&chipRev);
    if ((chipRev.major == 1) && (chipRev.minor == 1))
    {
      errataShift = 1;
    }
    
    /* Switching the CPU clock source to HFRCO */
    CMU_ClockSelectSet(cmuClock_HF,cmuSelect_HFRCO);
    
    /* Initialize DVK board register access */
    DVK_init();

    /* Connect ambient light sensor to EFM32 and disconnect potentiometer */
    DVK_disablePeripheral(DVK_POTMETER);
    DVK_enablePeripheral(DVK_AMBIENT);

    /* Connect accelerometer to EFM32. */
    DVK_enablePeripheral(DVK_ACCEL);
    
    /* Enable RS232 port B on the DVK */
    DVK_enablePeripheral(DVK_RS232B);
    
    /* Init POWER MANAGER*/
    vPMInit();
    
    /* Init ADC */
    initADC();
    
    /* Init DMA module */
    CMU_ClockEnable(cmuClock_DMA, true);
    
    /* Init Leuart module */
    initLeuart();
    
     /* Initialize board specific registers */
    VddCheckInit();

    /* Check if voltage is below 3V, if so use voltage boost */
    if (VddCheckLowVoltage(2.9))
      vboost = true;
  
    /* Disable Voltage Comparator */
    VddCheckDisable();

    /* Run Energy Mode with LCD */
    SegmentLCD_Init(vboost);
    
    /* Enable default characters on the sceen */
    SetInitialLCD();
    
    /* Initialize I2C driver, using standard rate */
    I2CDRV_Init(&i2cInit);
    
    /* Enable interrupts on pushbutton events only */
    DVK_disableInterrupt(0xffff);
    DVK_clearInterruptFlags(0xffff);
    DVK_enableInterrupt(BC_INTEN_PB);
    
    /* Enable GPIO for pushbuttons events*/
    GPIO_IRQInit();
    
    /* Enable AEM */
    setupSWO();
  
}

/* Function used by AEM */
void setupSWO(void)
{
  uint32_t *dwt_ctrl = (uint32_t *) 0xE0001000;
  uint32_t *tpiu_prescaler = (uint32_t *) 0xE0040010;
  uint32_t *tpiu_protocol = (uint32_t *) 0xE00400F0;

  CMU->HFPERCLKEN0 |= CMU_HFPERCLKEN0_GPIO;
  /* Enable Serial wire output pin */
  GPIO->ROUTE |= GPIO_ROUTE_SWOPEN;
  /* Set location 1 */
  GPIO->ROUTE = (GPIO->ROUTE & ~(_GPIO_ROUTE_SWLOCATION_MASK)) | GPIO_ROUTE_SWLOCATION_LOC1;
  /* Enable output on pin */
  GPIO->P[2].MODEH &= ~(_GPIO_P_MODEH_MODE15_MASK);
  GPIO->P[2].MODEH |= GPIO_P_MODEH_MODE15_PUSHPULL;
  /* Enable debug clock AUXHFRCO */
  CMU->OSCENCMD = CMU_OSCENCMD_AUXHFRCOEN;

  while(!(CMU->STATUS & CMU_STATUS_AUXHFRCORDY));

  /* Enable trace in core debug */
  CoreDebug->DHCSR |= 1;
  CoreDebug->DEMCR |= CoreDebug_DEMCR_TRCENA_Msk;

  /* Enable PC and IRQ sampling output */
  *dwt_ctrl = 0x400113FF;
  /* Set TPIU prescaler to 16. */
  *tpiu_prescaler = 0xf;
  /* Set protocol to NRZ */
  *tpiu_protocol = 2;
  /* Unlock ITM and output data */
  ITM->LAR = 0xC5ACCE55;
  ITM->TCR = 0x10009;
}


void vSampleLCD(portBASE_TYPE sample)
{
portBASE_TYPE i;
        /* Scale up the sample value for 2.5V */
        sample *= 5;
        sample /= 4;
  
        for (i = 0; i < 8; i++)
          SegmentLCD_ARing(i, 0);
        
        if ((sample >= 0) && (sample < 32))
             SegmentLCD_ARing(0, 1);
                       
        if ((sample >= 32) && (sample < 64)){
            for (i = 0; i < 2; i++)
              SegmentLCD_ARing(i, 1);     
        }
        
        if ((sample >= 64) && (sample < 96)){
            for (i = 0; i < 3; i++)
              SegmentLCD_ARing(i, 1);          
        }
        
        if ((sample >= 96) && (sample < 127)){
            for (i = 0; i < 4; i++)
              SegmentLCD_ARing(i, 1);          
        }      
        
        if ((sample >= 127) && (sample < 160)){
            for (i = 0; i < 5; i++)
              SegmentLCD_ARing(i, 1);          
        }     
  
        if ((sample >= 160) && (sample < 192)){
            for (i = 0; i < 6; i++)
              SegmentLCD_ARing(i, 1);          
        }
  
        if ((sample >= 192) && (sample < 224)){
            for (i = 0; i < 7; i++)
              SegmentLCD_ARing(i, 1);          
        }
  
        if ((sample >= 224) && (sample < 256)){
            for (i = 0; i < 8; i++)
              SegmentLCD_ARing(i, 1);          
        }
}

void SetInitialLCD()
{

    SegmentLCD_Symbol(LCD_SYMBOL_GECKO, 1);
    SegmentLCD_Symbol(LCD_SYMBOL_EFM32, 1);
    SegmentLCD_Battery(4);   
    SegmentLCD_EnergyMode(4, 1);
    SegmentLCD_EnergyMode(3, 1);
    SegmentLCD_EnergyMode(2, 1);
    SegmentLCD_EnergyMode(1, 1);
    SegmentLCD_EnergyMode(0, 1);

}

/**************************************************************************//**
 * @brief Update LCD with temperature
 * @param[in] temp Temperature to display.
 *****************************************************************************/
void temperatureUpdateLCD(TEMPSENS_Temp_TypeDef *temp)
{
  char text[8];
  TEMPSENS_Temp_TypeDef dtemp;
  
  /* Local copy */
  dtemp = *temp;

  memset(text, ' ', sizeof(text) - 1);
  text[sizeof(text) - 1] = 0;
  
  /* Celsius symbol */
  text[5] = 'C';

  /* Round temperature to nearest 0.5 */
  if (dtemp.f >= 0)
  {
    dtemp.i += (dtemp.f + 2500) / 10000;
    dtemp.f = (((dtemp.f + 2500) % 10000) / 5000) * 5000;
  }
  else
  {
    dtemp.i += (dtemp.f - 2500) / 10000;
    dtemp.f = (((dtemp.f - 2500) % 10000) / 5000) * 5000;
  }

  /* 100s */
  if (abs(dtemp.i) >= 100)
    text[0] = '0' + (abs(dtemp.i) / 100);

  /* 10s */
  if (abs(dtemp.i) >= 10)
    text[1] = '0' + ((abs(dtemp.i) % 100) / 10);

  /* 1s */
  text[2] = '0' + (abs(dtemp.i) % 10);

  /* 0.1s */
  text[3] = '0' + (abs(dtemp.f) / 1000);
   
  SegmentLCD_Write(text);
  SegmentLCD_Symbol(LCD_SYMBOL_DP4, 1);

  if ((dtemp.i < 0) || (dtemp.f < 0))
  {
    SegmentLCD_Symbol(LCD_SYMBOL_MINUS, 1);
  }
  else
  {
    SegmentLCD_Symbol(LCD_SYMBOL_MINUS, 0);
  }
  
}
