#include "PowerManager.h"
#include "efm32.h"
#include "task.h"
#include "croutine.h"
#include "efm32_rtc.h"
#include "efm32_adc.h"
#include "efm32_cmu.h"

#if ( configUSE_POWERMANAGER == 1 )

  /* Macro definitions */
	#define xPMCheckMode(mode) ((mode >= configSLEEPMODES ) || ( mode < 0) ? errINSERT_NOT_VALID_SLEEPMODE : pdTRUE)
	
  /* This array keeps track about how many tasks want to stay in each sleep mode*/
  PRIVILEGED_DATA static unsigned portCHAR ucSleepTasks [configSLEEPMODES] = {0};

  /* This array keeps track about how many "locked ISRs" want to stay in each sleep mode*/
  PRIVILEGED_DATA static volatile unsigned portCHAR ucSleepLockISR [configSLEEPMODES] = {0}; 
  
  /* This variable save the highest sleep mode requested by ISRs */
  PRIVILEGED_DATA static volatile unsigned portCHAR ucSleepISR; 
  
  /* Flag is 1 if an ISR done a request to move in a sleep mode. Otherwise flag is 0 */
  PRIVILEGED_DATA static volatile unsigned portCHAR ucFlagISR;  
  
 /* This value is used to keep update the CMU status. Since when MCU goes in sleep
  * mode the value inside CMU->STATUS can suffer a misleading value. Then is a 
  * responsability of PM to keep update this value. 
  * This variable is only used for Energy Micro 
  */
  PRIVILEGED_DATA static volatile portBASE_TYPE xPMStatus;
  
  /* ------ Internal functions used by Power Manager for porting EFM32 in FreeRTOS ------ */
	static portBASE_TYPE xPMGoToSleepMode ( unsigned portCHAR xMode ) PRIVILEGED_FUNCTION;
    static void vPMGoToSleepModeEM1( void ) PRIVILEGED_FUNCTION;
	static void vPMGoToSleepModeEM2( void ) PRIVILEGED_FUNCTION;
	static void vPMGoToSleepModeEM3( void ) PRIVILEGED_FUNCTION;
	static void vPMGoToSleepModeEM4( void ) PRIVILEGED_FUNCTION;
	static void vPMRestoreSleepMode( void ) PRIVILEGED_FUNCTION;
  
  #if ( configTICKLESS == 1 )  
  
    #define portNVIC_INT_CTRL		(( volatile unsigned long *) 0xe000ed04)
    #define portNVIC_PENDSVSET		0x10000000
    #define RTC_CTRL_EN_DISABLE         0x1UL
 
    /* This variable stores the tick to awake the MCU from tickless mode*/
    PRIVILEGED_DATA static volatile portTickType TickToWakeup;
  
    /* When this variable is 1 the RTC knows the time to awake itself */
    PRIVILEGED_DATA static volatile portCHAR xTickPass = 0;
    
    /* These variables keep track of ticks that occur when ISRs keep busying
       the MCU in tickless mode
     */    
    PRIVILEGED_DATA static volatile portTickType CounterBefore, Counter = 0;
        
    /* When this variable is 1 means that Tickless is occuring */
    PRIVILEGED_DATA static volatile portCHAR xPMTicklessENB = 0;

    /* This function checks if it is worthwhile to use a tickless mode or not */
    static portBASE_TYPE xPMBreakEvenTime ( portTickType xNextTickTask, portTickType xNextTickCoRoutine) PRIVILEGED_FUNCTION;

    /* This function sets the next wake up time when MCU is running in tickless mode */
    static void vPMSetRTC ( portTickType TimetoWakeup ) PRIVILEGED_FUNCTION;

  #endif
/****************************************************************************
 * @brief
 *  Initialise Power Management Framework.
 *
 * @details
     This function initialise internal registers used to manage efficiently GPMF.
 * @note
     This function must be invoked before the scheduler starts. For EFM32 it is
	 necessary to use it.
 ******************************************************************************/
  
  void vPMInit ( void )
  {
      /*Lock EMU function*/
      EMU->LOCK = EMU_LOCK_LOCKKEY_LOCK;
      
      /* Set SEVONPEND in CORTEX core */
      SCB->SCR |= (1 << SCB_SCR_SEVONPEND_Pos);

  }
  
/*******************************************************************************
 * @brief
 *  Set a sleep mode in tasks' body.
 *
 * @details
     This functions provides to set a sleep mode during task execution. If the 
     sleep mode passed as input it is not a valid value, then the function returns
     an error value errINSERT_NOT_VALID_SLEEPMODE. 
	 Otherwise it returns a valid value pdTRUE.
 *
 * @note
     This function can only be invoked in tasks body and not inside ISRs.

 * @par
     Get in input the value of sleep mode to go in.
 ******************************************************************************/
  portBASE_TYPE xPMSetSleepMode ( unsigned portCHAR xMode)
  {
	portBASE_TYPE xReturn;
	
	portENTER_CRITICAL();
	/*Check if the value given in input is valid*/
	if ( xPMCheckMode( xMode ) == pdTRUE){
	   ucSleepTasks[ucTaskGetCurrentTCB()]--;
           vTaskPutCurrentTCB(xMode);
           ucSleepTasks[xMode]++;
	   xReturn = pdTRUE;
	}else{
           xReturn = errINSERT_NOT_VALID_SLEEPMODE;
	}
	portEXIT_CRITICAL();
	
	return xReturn;
  }

/****************************************************************************
 * @brief
 *   Set a sleep mode in ISRs' body.
 *
 * @details
    This functions provides to set a sleep mode during ISR execution. If the 
    sleep mode passed as input it is a not valid value, then the function returns
    an error value errINSERT_NOT_VALID_SLEEPMODE. 
	Otherwise it returns a valid value pdTRUE.
 *
 * @note
    The sleep mode passed as input is valid only until the system meets the first
    idle task. For the following idle tasks the sleep mode is being discarded.
    This function can only be invoked in ISR's body.
 * @par
    Get as input the value of sleep mode to go in
 ******************************************************************************/
  portBASE_TYPE xPMSetSleepModeFromISR ( unsigned portCHAR xMode )
  {
	portBASE_TYPE xReturn;
	
	/*Check if the value given in input is valid*/
	if ( xPMCheckMode( xMode ) == pdTRUE){
           ucSleepISR = xMode;
	   ucFlagISR = 1;
	   xReturn = pdTRUE;
	}else
           xReturn = errINSERT_NOT_VALID_SLEEPMODE;
	
	return xReturn; 
  }

/****************************************************************************
 * @brief
 *  Get the current sleep mode where the MCU can be moved in.  
 *
 * @details
     According to requests done with functions like xPMSetSleepModeFromISR or 
     xPMSetSleepMode the function aims to know in which sleep mode the MCU can be
     moved in. The function returns the value of current sleep mode.
 *
 * @note
     This function can be invoked only in tasks' body. 

 ******************************************************************************/
  
  unsigned portCHAR ucPMGetSleepMode ( void )
  {
    unsigned portCHAR i, ucReturn = configDEFAULT_SLEEPMODE;

    portENTER_CRITICAL();
    
   /*Check which is the suitable mode between tasks and LockISRs*/
    for (i=0; i<configSLEEPMODES; i++)
	if ( (ucSleepTasks[i] > 0) || (ucSleepLockISR[i] > 0)){
	   ucReturn = i;
	   break;
        }
    
   /*Check which is the suitable mode among simple ISR, tasks and LockISR */
	if ( ( ucFlagISR) && ( ucReturn > ucSleepISR))
	   ucReturn = ucSleepISR;
          
    portEXIT_CRITICAL();
	
    return ucReturn;
  }

/****************************************************************************
 * @brief
 *  Get the current sleep mode where the MCU can be moved in (FromISR).  
 *
 * @details
     According to requests done with functions like xPMSetSleepModeFromISR or 
     xPMSetSleepMode the function aims to know in which sleep mode the MCU can be
     moved in. The function returns the value of current sleep mode.
 *
 * @note
     This function can be invoked only in ISRs' body. 
 ******************************************************************************/
  
  unsigned portCHAR ucPMGetSleepModeFromISR (void)
  {
    unsigned portCHAR i, ucReturn = configDEFAULT_SLEEPMODE;
	
      /*Check which is the suitable mode between tasks and LockISRs*/
	  for (i=0; i<configSLEEPMODES; i++)
	    if ( (ucSleepTasks[i] > 0) || (ucSleepLockISR[i] > 0)){
	       ucReturn = i;
	       break;
		}
	  /*Check which is the suitable mode among simple ISR, tasks and LockISR */
	  if ( ( ucFlagISR) && ( ucReturn > ucSleepISR))
	      ucReturn = ucSleepISR;
		
    return ucReturn;
  }

/****************************************************************************
 * @brief
 *  This function locks a sleep mode inside an ISR.
 *
 * @details
     The sleep mode is locked if there is a need to keep the value of sleep mode
     for a while. The way to unlock this value is to use the following functions
     xPMUnlockSleepModeFromISR or xPMUnlockSleepMode.
 *
 * @note
     This function can be invoked only in ISRs' body. Besides, this function 
     can be used as an alterantive solution to xPMSetSleepModeFromISR.
	 Since the use of this function can involve in a misleading use of 
     sleep modes, then the function must be used only when it is necessary.
     The function can lock more sleep modes and also nested ones.
 * @par  
     Get in input the value of sleep mode to lock. 
 ******************************************************************************/
  
  portBASE_TYPE xPMLockSleepModeFromISR ( unsigned portCHAR xMode)
  {
    /*Check if the value given in input is a valid one*/
    if ( xPMCheckMode( xMode ) == pdTRUE){
       ucSleepLockISR[xMode]++;
	  return pdTRUE;
       }else 
	  return errINSERT_NOT_VALID_SLEEPMODE;
  }

  /****************************************************************************
 * @brief
 *  This function unlocks a sleep mode previously locked with xPMLockSleepModeFromISR.
 *
 * @details
     This function unlocks a sleep mode only inside inside an ISR's body. If the function 
     tries to unlock a sleep mode not locked previously, then the function 
     discards the request and returns an error code. Otherwise it returns a 
     valid number pdTRUE.
 *
 * @note
     Function to call only inside ISR.
 * @par
     Get in input the value of sleep mode to unlock. 
 ******************************************************************************/
  
  portBASE_TYPE xPMUnlockSleepModeFromISR (unsigned portCHAR xMode)
  {
    /*Check if the value given in input is a valid one*/
     if ( xPMCheckMode( xMode ) == pdTRUE){
    /*If a request is issued and its value is equal zero, it means that there is no ISR to unlock*/
	 if (ucSleepLockISR[xMode] == 0)
	    return errCOULD_NOT_UNLOCK_SLEEPMODE;
	 else{
            ucSleepLockISR[xMode]--;
            return pdTRUE;
	 }
     }else 
       return errINSERT_NOT_VALID_SLEEPMODE; 
  }

/****************************************************************************
 * @brief
 *  This function unlocks a sleep mode previously locked with xPMLockSleepModeFromISR
 *
 * @details
     This function unlocks a sleep mode only inside an task's body. If the function 
     tries to unlock a sleep mode not locked previously, then the function 
     discards the request and returns an error code. Otherwise it returns a 
     valid number pdTRUE.
 *
 * @note
     Function to call only inside task.
 * @par
     Get in input the value of sleep mode to unlock. 
 ******************************************************************************/
  
  portBASE_TYPE xPMUnlockSleepMode (unsigned portCHAR xMode)
  {
    portBASE_TYPE xReturn;
	
       portENTER_CRITICAL();
       
       /*Check if the value given in input is a valid one*/ 
       if ( xPMCheckMode( xMode ) == pdTRUE){
          /*If a request is issued and its value is equal zero, it means that there is no ISR to unlock*/
          if (ucSleepLockISR[xMode] == 0) 
	     xReturn = errCOULD_NOT_UNLOCK_SLEEPMODE;
	  else{
	     ucSleepLockISR[xMode]--;
	     xReturn= pdTRUE;
	  }
       }else 
	   xReturn = errINSERT_NOT_VALID_SLEEPMODE; 
	   
       portEXIT_CRITICAL();	   
	
    return xReturn;
  }
  
/****************************************************************************
 * @brief
 *  This function is used in IDLE TASK and can be used in two possible ways: 
	POWER MANAGER: It only pushes the MCU in sleep modes;
	TICKLESS MODE: It uses either Tickless and POWER MANAGER features;  
 *
 * @details
    This function is directly called inside the IDLE TASK. The user DOESN NOT 
    have to call it in vApplicationIdleHook.
 *
 * @note
    If the user does not use the vPMInit, then ucPMIdleSleepMode does not 
    work properly. 
 ******************************************************************************/
  
void vPMIdleSleepMode ( void )
{
    unsigned portCHAR sleep;
    portTickType TickNextTask;
    portTickType TickNextRoutine = portMAX_DELAY;
      
  #if ((configTICKLESS == 1)  && (configUSE_POWERMANAGER == 1)) 
    
      if ( xTickPass == 0){ 
        
          /* Make sure the eventregister is set to 0. Otherwiswe core keep running*/
          __SEV();
          __WFE();
          
          /* Suspend either tasks and Interrupts */
          vTaskSuspendAll(); 
          portENTER_CRITICAL();  
          
          /* Tickless mode is starting*/
          xPMTicklessENB = TICKRUNNING;
          CounterBefore = 0;
          Counter = 0;
          
          /* Retrieve the sleep mode to use in idle state */
          sleep = ucPMGetSleepMode();
          
          /* RTC does not work beyond EM2. So, I have to force EM2 as max sleep mode */
          if (sleep > EM2)
            sleep = EM2;
          
          TickNextTask = xTaskNextTick();
          #if (configUSE_CO_ROUTINES == 1)
              TickNextRoutine = xCoRoutineNextTick();
          #endif
          
          SegmentLCD_EnergyMode(0, 0);
          if (sleep == EM2)
            SegmentLCD_EnergyMode(1, 0);
            
          /* Break even checks if it is worthwhile to use tickless mode */
          if ( (xPMBreakEvenTime(TickNextTask,TickNextRoutine) == pdPASS) ){	
            
				#if (configUSE_CO_ROUTINES == 1)
					if ( TickNextTask <= TickNextRoutine )
					   TickToWakeup = (TickNextTask - xTaskGetTickCountTickless()) - 1;
					else 
					   TickToWakeup = (TickNextRoutine - xTaskGetTickCountTickless()) - 1;
				#else
					TickToWakeup = (TickNextTask - xTaskGetTickCountTickless()) - 1;
				#endif
			 
				#if (configPRESC == 0)
					/* No prescaling */
					TickToWakeup = TickToWakeup * (RTC_FREQ/configTICK_RATE_HZ);
				#endif
				
                if ( TickToWakeup > MAXRTCOUNT )
                        TickToWakeup = MAXRTCOUNT;
                                        
				vPMSetRTC(TickToWakeup);
				xPMGoToSleepMode(sleep);  /* Go in sleep mode */
				/* Tickless starts right now */
				xPMTicklessENB = TICKLESS;
          }else{
				/* No tickless mode, I stay in sleep mode until next tick occurs */
				vPMSetRTC(((RTC_FREQ/PRESC)/configTICK_RATE_HZ));
				RTC_IntClear(RTC_IFC_COMP0);
				NVIC_ClearPendingIRQ(RTC_IRQn);
				xTickPass = 1;
				xPMGoToSleepMode(sleep);  /* Go in sleep mode */
          }    
          
          /* Reset ucFlagISR if SSM issues a request before idle task */ 
          ucFlagISR = 0;
          
          /* Re-enable either interrupts and tasks */
          portEXIT_CRITICAL(); 
          xTaskResumeAll();
      } 
  #endif

  #if ((configTICKLESS == 0)  && (configUSE_POWERMANAGER == 1))  
      
      vTaskSuspendAll();   
      portENTER_CRITICAL();    
      
      /* Retrieve the sleep mode to use for idle task */
      sleep = ucPMGetSleepMode();
      xPMGoToSleepMode(sleep); /* Go in sleep mode */
      /* Reset ucFlagISR if SSM issues a request before idle task */ 
      ucFlagISR = 0;
      
      portEXIT_CRITICAL(); 
      xTaskResumeAll();
  #endif
      
}
 
/****************************************************************************
 * @brief
 * This function is a porting for Energy Micro. Besides, it provides to move the
   MCU in the sleep mode received as input.
 *
 * @details
      According to the value received as input, the function enables the MCU in 
      the right sleep mode. If the sleep mode got as input is not valid, then 
      the function returns an error value (-1).
 *
 * @par
     Get in input the value of sleep mode. 
 ******************************************************************************/
  
 static portBASE_TYPE xPMGoToSleepMode ( unsigned portCHAR xMode )
 {
    switch(xMode)
	{
          case RUNNING:
            break;
          case EM1:
            vPMGoToSleepModeEM1();
            break;
          case EM2:
            vPMGoToSleepModeEM2();
            break;
          case EM3:
            vPMGoToSleepModeEM3();
            break;
          case EM4:
            vPMGoToSleepModeEM4();
            break;
          default:
            return errCOULD_NOT_GOTO_SLEEPMODE;
	} 
 return pdTRUE;
 }

/****************************************************************************
 * @brief
 *   This function is a porting for Energy Micro. Move the MCU in EM1
 *
 * @details
      The function before to go in EM1 it sets the CTRL register and it 
      unlocks the EMU register locked in vPMInit.
 *
 * @note
      The MCU is awakened with an event (WFE)
 ******************************************************************************/
 
static void vPMGoToSleepModeEM1( void )
{
  /*Unlock EMU registers*/
  EMU->LOCK = EMU_LOCK_LOCKKEY_UNLOCK;
  
  /*It is used to gurantee that the user can not change EMs in the rest of program*/
  EMU->CTRL |= EMU_CTRL_EM2BLOCK;
  
  /*If EMU register was locked, then lock it again */
  EMU->LOCK = EMU_LOCK_LOCKKEY_LOCK;

   
  /* Enter Cortex-M3 sleep mode */
  SCB->SCR &= ~SCB_SCR_SLEEPDEEP_Msk;
  __WFE();


}

/****************************************************************************
 * @brief
 *   This function is a porting for Energy Micro. Move the MCU in EM2
 *
 * @details
      The function before to go in EM2 it REMOVES the CTRL register and it 
      unlocks the EMU register locked in vPMInit. Besides, it keeps updated the 
      value of CMU->STATUS (explained above). When the MCU is woken up it enables 
	  all oscillators that were enabled before to go in a sleep mode.
 *
 * @note
      The MCU is awakened with an event (WFE)
 ******************************************************************************/

static void vPMGoToSleepModeEM2( void )
{

    /*Save the content of STATUS before to move in EM2. This is useful because the HW can alter 
      the value of STATUS register in the meanwhile when the MCU is in EM2 */
    xPMStatus = (CMU->STATUS);
	
    /* Unlock EMU registers */
    EMU->LOCK = EMU_LOCK_LOCKKEY_UNLOCK;
    
    /*Remove the flag to block EM2 and lower (if any)*/
    EMU->CTRL &= ~EMU_CTRL_EM2BLOCK;
    
    /*If EMU register was locked, then lock it again */
    EMU->LOCK = EMU_LOCK_LOCKKEY_LOCK;

    /* Enter Cortex-M3 deep sleep mode */
    SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;    
    __WFE();

}

/****************************************************************************
 * @brief
 *   This function is a porting for Energy Micro. Move the MCU in EM3
 *
 * @details
      The function before to go in EM1 it REMOVES the CTRL register and it also
      unlocks the EMU register locked in vPMInit. Besides, it keeps updated the 
      value of CMU->STATUS (explained above). When the MCU is woken up it enables 
	  all oscillators that were enabled before to go in a sleep mode.
 *
 * @note
      The MCU is awakened with an event (WFE). The only difference 
      with EM2 is that the function takes care about CMU->LOCK register, if this 
      register was enabled before sleep, then it sets the register again with LOCKED. 
 ******************************************************************************/

static void vPMGoToSleepModeEM3( void )
{
  portBASE_TYPE xPMStatusLockCMU;

    /* Save the content of STATUS before to move in EM2. This is useful because the HW can alter 
    the value of STATUS register in the meanwhile when the MCU is in EM2 */
    xPMStatus = (CMU->STATUS);
	
    /* Unlock EMU registers */
    EMU->LOCK = EMU_LOCK_LOCKKEY_UNLOCK;
    
    /*Remove the flag to block EM2 and lower (if any)*/
    EMU->CTRL &= ~EMU_CTRL_EM2BLOCK;
    
    /*If EMU register was locked, then lock it again */
    EMU->LOCK = EMU_LOCK_LOCKKEY_LOCK;

    /* CMU registers may be locked */
    xPMStatusLockCMU = CMU->LOCK & CMU_LOCK_LOCKKEY_LOCKED;
    CMU->LOCK = CMU_LOCK_LOCKKEY_UNLOCK;

    /* Disable LF oscillators */
    CMU->OSCENCMD = CMU_OSCENCMD_LFXODIS | CMU_OSCENCMD_LFRCODIS;
	
    /*If CMU register was locked, then lock it again */
    if ( xPMStatusLockCMU )
      CMU->LOCK = CMU_LOCK_LOCKKEY_LOCK;
	  
    /* Enter Cortex-M3 deep sleep mode */
    SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;
    __WFE();
	
    vPMRestoreSleepMode();
	        
}

/****************************************************************************
 * @brief
 *   This function is a porting for Energy Micro. Move the MCU in EM4
 *
 * @details
      The function moves the MCU directly in EM4. Only a reset can awake the MCU.
 *
 * @note
      
 ******************************************************************************/
static void vPMGoToSleepModeEM4( void )
{
  unsigned portCHAR i;
  
   /* Unlock EMU registers */
   EMU->LOCK = EMU_LOCK_LOCKKEY_UNLOCK;

   for (i = 0; i < 4; i++)
   {
     EMU->CTRL = (2 << _EMU_CTRL_EM4CTRL_SHIFT);
     EMU->CTRL = (3 << _EMU_CTRL_EM4CTRL_SHIFT);
   }
   EMU->CTRL = (2 << _EMU_CTRL_EM4CTRL_SHIFT);
}
/****************************************************************************
 * @brief
 *   This function is used to restore the oscillators after EM2 or EM3
 *
 * @details
      The functions restore those oscillators were enabled before to move the 
      MCU in sleep mode. This operation is accomplished thanks to xPMStatus that 
      keeps track which oscialltors were enabled before to go in sleep mode.
 *
 * @note
      The HFRCO core is automatically restored by the system.
 ******************************************************************************/
static void vPMRestoreSleepMode( void )
{
  portBASE_TYPE xPMStatusLockCMU;

  /* CMU registers may be locked */
  xPMStatusLockCMU = CMU->LOCK & CMU_LOCK_LOCKKEY_LOCKED;
  CMU->LOCK = CMU_LOCK_LOCKKEY_UNLOCK;

  /* Restore oscillators used before to drop in EM2/EM3 */
  CMU->OSCENCMD = xPMStatus & (CMU_STATUS_AUXHFRCOENS | CMU_STATUS_HFXOENS | CMU_STATUS_LFRCOENS | CMU_STATUS_LFXOENS);

  /* If the oscillator used for clocking core is not HFRCO, then I restore the right one */
  switch (xPMStatus & (CMU_STATUS_HFXOSEL | CMU_STATUS_HFRCOSEL | CMU_STATUS_LFXOSEL | CMU_STATUS_LFRCOSEL))
  {
      case CMU_STATUS_LFRCOSEL:
        /* Wait for LFRCO to stabilize */
        while (!(CMU->STATUS & CMU_STATUS_LFRCORDY)) ;
        CMU->CMD = CMU_CMD_HFCLKSEL_LFRCO;
        break;
    
      case CMU_STATUS_LFXOSEL:
        /* Wait for LFXO to stabilize */
        while (!(CMU->STATUS & CMU_STATUS_LFXORDY)) ;
        CMU->CMD = CMU_CMD_HFCLKSEL_LFXO; 
        break;
    
      case CMU_STATUS_HFXOSEL:
        /* Wait for HFXO to stabilize */
        while (!(CMU->STATUS & CMU_STATUS_HFXORDY)) ;
        CMU->CMD = CMU_CMD_HFCLKSEL_HFXO;
        break;
    
      default: /* CMU_STATUS_HFRCOSEL */
        /* If core clock was HFRCO core clock, it is automatically restored by system */
        break;
  }

  /* Restore CMU register locking */
  if ( xPMStatusLockCMU )
    CMU->LOCK = CMU_LOCK_LOCKKEY_LOCK;
  
}

/****************************************************************************
 * @brief
 *   This function is internally used by Power Manager and can not be used in 
     application level.
 *
 * @details
     This function updates the number of tasks that stay in the sleep mode 
     given as input. It increments the number of tasks in ucSleepTasks array.
 *      
 ******************************************************************************/
void vPMUpdateSleepModesInc (unsigned portCHAR xMode )
{
   ucSleepTasks[xMode]++;
}

/****************************************************************************
 * @brief
 *   This function is internally used by Power Manager and can not be used in 
     application level.
   @ret
     Return pdTRUE if the referred sleep mode has been decremented succesfully
     Return pdFAIL if the referred sleep mode did not contain any task
 * @details
     This function updates the number of tasks that stay that sleep mode 
     given as input. It decrements the number of tasks in ucSleepTasks array.
 *      
 ******************************************************************************/
portBASE_TYPE xPMUpdateSleepModesDec ( unsigned portCHAR xMode )
{
  if (ucSleepTasks[xMode] == 0)
      return pdFAIL;
  else{
      ucSleepTasks[xMode]--;
      return pdTRUE;
  }
}
#endif

#if ( configTICKLESS == 1 )

/****************************************************************************
 * @brief
 *  Handler used by RTC tick timer. It is used to increment the internal FreeRTOS
    tick timer. 
 * @details
    The handler sets a new tick every time is being called. Besides, it also updates
    the co-routine module (if enabled).
*      
 ******************************************************************************/
void RTC_IRQHandler  ( void )
{
  unsigned long ulDummy;
  
     if ( RTC->IF & RTC_IF_COMP0 ){ 
         
         if (( xPMTicklessENB == TICKLESS )){

            #if (configPRESC == 1)
		vTaskUpdateTickCountFromISR(TickToWakeup);	
	    #else
		vTaskUpdateTickCountFromISR(TickToWakeup/(RTC_FREQ/configTICK_RATE_HZ));	
	    #endif
				
            #if (configUSE_CO_ROUTINES == 1)
               /* I keep update the coroutine tick. Thus, I avoid to perform 
                    * (waste time and energy) all missed ticks when coroutine-
                    * scheduling is being called.
                    */
                    prvCheckDelayedListTickless(); 
            #endif
         }
         
         /* Set next tick time */
         RTC_IntClear(RTC_IFC_COMP0);
         RTC_CompareSet(0,((RTC_FREQ/PRESC)/configTICK_RATE_HZ)); 

         /* If using preemption, also force a context switch. */
         #if configUSE_PREEMPTION == 1
                *(portNVIC_INT_CTRL) = portNVIC_PENDSVSET;
         #endif
         
         if (xPMTicklessENB != TICKLESSNOPREEMPTED)
         {
             ulDummy = portSET_INTERRUPT_MASK_FROM_ISR();
             {
                   vTaskIncrementTick();
             }
             portCLEAR_INTERRUPT_MASK_FROM_ISR( ulDummy );   
         }     
         
         /* Reset the tickless variable in running mode */         
         xPMTicklessENB = TICKRUNNING;
         xTickPass = 0;
     }	

}

/****************************************************************************
 * @brief
 *  The function checks if it is worthwhile to use a Tickless mode or a normal 
    tick timer.  
 *
 * @details
     The parameters used to evaluate if break even time is satisfied are:
  
      *Tick to wait: If the number of ticks before next task are less than two
                     the tickless is useless. Because if an ISR occurs in the 
                     meanwhile when the MCU is an tickless mode, it delays the 
                     internal ticks. 
                
     Thus, if the above condition is not satisfied it is better to use a normal
     tick timer. 
 *
 * @ret
      Function returns
	@pdPASS BreakEvenTime is satisfied and it uses a Tickless mode;
	@pdFAIL BreakEvenTime is not satisfied and MCU keeps using tick timer;
    
 * @par
	 @xNextTickTask Time when next task must be scheduled;
	 @xNextTickCoRoutine Time when next co-routine must be scheduled; 
 ******************************************************************************/
 
static portBASE_TYPE xPMBreakEvenTime ( portTickType xNextTickTask, portTickType xNextTickCoRoutine){
	
	portCHAR NumTickBreakEven;
	
	/* If xNextTickTask or xNextTickCoRoutine is equal 0. It means that a task 
	 * or co-routine needs to run with same priority of idle task (or pending).
	 * Thus, we cannt go in sleep mode. Break even time is not satisfied.
	 */
        
        #if configUSE_CO_ROUTINES == 1
            if (( xNextTickTask == 0) || ( xNextTickCoRoutine == 0 ))
                return pdFAIL; 
        #else
            if ( xNextTickTask == 0 )
                return pdFAIL; 
        #endif
               
        NumTickBreakEven = MinTICK; /* Mimimum ticks required  to use tickless mode */
        
        /* If the number ticks for a task or a co-routine is less than break even
         * condition (sum of all factors) the MCU does not use Tickless mode. 
         */
        #if configUSE_CO_ROUTINES == 1
            if (( NumTickBreakEven >=  ( xNextTickTask - xTaskGetTickCountTickless())) || ( NumTickBreakEven >= ( xNextTickCoRoutine - xTaskGetTickCountTickless())))  
                return pdFAIL;
            else
                return pdPASS;
        #else
            if ( NumTickBreakEven >= ( xNextTickTask - xTaskGetTickCountTickless()))  
                return pdFAIL;
            else
                return pdPASS;		
        #endif
	
    return pdFAIL;
}

/****************************************************************************
 * @brief
 * This functions sets the RTC timer to awake the core up when next tick occurs.
 * The RTC overflows every 1,52 days, so the core can stay in sleeping mode at 
 * most for this amount of time. This function determines the Tickless mode.
 ******************************************************************************/

static void vPMSetRTC ( portTickType TimetoWakeup ){
  
    /* Reset Counter. If RTC->CNT > CTRL->COMP0TOP overflow occurs.
     * Thus, I reset the counter before to set new CTRL->COMP0TOP
     */
    RTC->CTRL &= ~(RTC_CTRL_EN_DISABLE);
    while (RTC->SYNCBUSY & RTC_SYNCBUSY_CTRL); 
    
    /* When timetowake == 0 means that the core must sleep only 1 time
       because I decremented -1 when I issued a request for xCoRoutineNextTick
       and xTaskNextTick.
    */
    if (TimetoWakeup == 0)
      TimetoWakeup ++;
          
    /* Set the time to awake the core from sleeping mode */
    RTC_CompareSet(0,TimetoWakeup);  

    /* Start Counter of RTC to wake up the core*/
    RTC->CTRL |= RTC_CTRL_EN;
    while (RTC->SYNCBUSY & RTC_SYNCBUSY_CTRL);   
    
              RTC_IntClear(RTC_IFC_COMP0);
          NVIC_ClearPendingIRQ(RTC_IRQn);
          
}

/****************************************************************************
 * @brief
* This function is used to guarantee that time is being keeping updated also
  when ISRs are executed in Tickless mode. Thus, when the core returns back to
  running mode it knows the precise amount of time spent in tickless mode.
  Besides, the function can be also used as portEND_SWITCHING_ISR defined in 
  FreeRTOS.

 * @details
     This function issues a context switching if the parameter given as input is 
     equal to pdTRUE. Moreover, it also handles the three states inside Tickless
     mode. The possible states are:
  
        *Tickless running: Tickless has been defined and the core is waiting the 
                           time to wake up.

        *Tickless preempted: During tickless an ISR generates a request to 
                             preempt the scheduler because a task has been moved 
                             in ready list.

        *Tickless No preempted: During tickless an ISR does not generates a request 
                                to preeempt the scheduler.

     The main benefit of this function is to keep updated the xTickCount variable
     when many ISRs occur after sleeping the core. If all ISRs take a long time,
     this function updates xTickCount with the number of ticks lost.

 * @par
     @xHigherPriorityTaskWoken 
	 @pdTRUE Context switching is required;
	 @pdFALSE Context switching is not required; 
 ******************************************************************************/

void vPMTicklessFromISR( portBASE_TYPE xHigherPriorityTaskWoken){

  portTickType Add = 0, temp;
  
  /* If tickless is running I need to check if context switched is required */
  if ( xPMTicklessENB != TICKRUNNING){
    
      /* Context switching is required */
      if  (xHigherPriorityTaskWoken == pdTRUE){
          /* This section calculates the time elapsed since the first ISR occured. 
             Time is stored in Add. This value is immediately added in xTickCount.
          */
          switch (xPMTicklessENB){
            
              case TICKLESS:
					  #if configPRESC == 1
						Add = RTC_CounterGet();
					  #else
						Add = (RTC_CounterGet()/(RTC_FREQ/configTICK_RATE_HZ));
					  #endif
                      break;
                      
              case TICKLESSNOPREEMPTED: 
					  #if configPRESC == 1
						Add = RTC_CounterGet() - CounterBefore;
					  #else
                        Add = (RTC_CounterGet()/(RTC_FREQ/configTICK_RATE_HZ)) - CounterBefore;
					  #endif			  
                      CounterBefore = 0;
                      break;
        
              case TICKLESSPREEMPTED: /* Number of ticks elapsed since last check*/
					  #if configPRESC == 1
						Add = RTC_CounterGet() - CounterBefore;
					  #else
                        Add = (RTC_CounterGet()/(RTC_FREQ/configTICK_RATE_HZ)) - CounterBefore;
					  #endif	
                      CounterBefore = 0;
                      break;
                      
              default: Add = 0;
          }    
          
          /* If using preemption, also force a context switch. */
          #if configUSE_PREEMPTION == 1
                *(portNVIC_INT_CTRL) = portNVIC_PENDSVSET;
          #endif
          
          /* Now the RTC behaves as a normal tick timer. It does not work in a
             tickless mode anymore.
           */
          if ( ((xPMTicklessENB == TICKLESS) || (xPMTicklessENB == TICKLESSNOPREEMPTED)) ){
              xPMTicklessENB = TICKLESSPREEMPTED;  /* Now the system runs in preempted mode */
              
              /* Reset Counter of RTC again */
              RTC->CTRL &= ~(RTC_CTRL_EN_DISABLE);
              while (RTC->SYNCBUSY & RTC_SYNCBUSY_CTRL);     
              
              /* Set RTC as tick timer */ 
              RTC_CompareSet(0,((RTC_FREQ/PRESC)/configTICK_RATE_HZ));  
                
              /* Start Counter of RTC again */
              RTC->CTRL |= RTC_CTRL_EN;
              while (RTC->SYNCBUSY & RTC_SYNCBUSY_CTRL);    
          }
         
          /* I clear all pending interrupt because I have already updated the
             variable of xTickCount. */
          RTC_IntClear(RTC_IFC_COMP0);
          NVIC_ClearPendingIRQ(RTC_IRQn);
      }
      
      /* If context switching is not required I only have to keep xTickCount updated */
      if ( xHigherPriorityTaskWoken == pdFALSE){
          
          /* I check the number of ticks lost during execution of ISRs */
          if ( (xPMTicklessENB == TICKLESS)||(xPMTicklessENB == TICKLESSNOPREEMPTED) ){
		  
		      #if (configPRESC == 1)
				 temp = RTC_CounterGet();
			  #else
				 temp = (RTC_CounterGet()/(RTC_FREQ/configTICK_RATE_HZ));
			  #endif
			  
              if ( Counter == 0)
                 Add = temp;
              else
                 Add = temp - CounterBefore;
              
              /* Right now the system runs in nopreempted tickless mode */
              xPMTicklessENB = TICKLESSNOPREEMPTED;
              Counter ++;
              CounterBefore = temp;
          }
          
          /* If a previous ISR was running in PREEMPTED TICKLESS MODE, I keep
             counting the number of ticks lost after last executed ISR. 
           */
          if ( xPMTicklessENB == TICKLESSPREEMPTED ){     
			  #if configPRESC == 1
				Add = RTC_CounterGet() - CounterBefore;
			  #else
				Add = (RTC_CounterGet()/(RTC_FREQ/configTICK_RATE_HZ)) - CounterBefore;
			  #endif	
              CounterBefore = 0;
          }
      }
      /* Update the number of 'ticks lost' in xTickCount variable*/
      vTaskUpdateTickCountFromISR(Add);
  }else
    { 
     /* This section issues a context switching when we use this function as 
        portEND_SWITCHING_ISR.
      */
        if ((xHigherPriorityTaskWoken != pdFALSE)){
            /* If using preemption, also force a context switch. */
            #if configUSE_PREEMPTION == 1
                *(portNVIC_INT_CTRL) = portNVIC_PENDSVSET;
            #endif
        }
  }
    
}
#endif
 